using OSMDatastructure;
using OSMDatastructure.Graph;
using OSMImporter;

namespace Pathfinding;

public static partial class Pathfinder
{
    public static List<PathNode> AStarDistance(string workingDir, Coordinates start,
        Coordinates goal)
    {
        RegionManager regionManager = new (workingDir);
        ValueTuple<OsmNode?, OsmNode?> startAndEndNode = SetupNodes(start, goal, regionManager);
        if (startAndEndNode.Item1 is null || startAndEndNode.Item2 is null)
            return new List<PathNode>();
        OsmNode goalNode = startAndEndNode.Item2!;

        PriorityQueue<OsmNode, double> toVisit = new();
        toVisit.Enqueue(startAndEndNode.Item1, 0);
        bool stop = false;

        while (toVisit.Count > 0)
        {
            OsmNode closestNodeToGoal = toVisit.Dequeue();

            foreach (OsmEdge edge in closestNodeToGoal.edges)
            {
                OsmNode? neighbor = regionManager.GetNode(edge.neighborId, edge.neighborRegion);
                if (neighbor is not null)
                {
                    double newPotentialWeight =
                        closestNodeToGoal.currentPathWeight + Utils.DistanceBetween(closestNodeToGoal, neighbor);
                    if (newPotentialWeight < neighbor.currentPathWeight)
                    {
                        neighbor.previousPathNode = closestNodeToGoal;
                        neighbor.currentPathWeight = newPotentialWeight; //for other types of routing
                        neighbor.currentPathLength = newPotentialWeight;
                        neighbor.directDistanceToGoal = Utils.DistanceBetween(neighbor, goalNode);

                        if (neighbor.Equals(goalNode))
                        {
                            stop = true;
                        }
                        else if(!stop)
                        {
                            toVisit.Enqueue(neighbor, neighbor.directDistanceToGoal);
                        }
                    }
                }
            }
        }

        List<PathNode> path = new();
        OsmNode? currentNode = goalNode;
        while (currentNode is not null)
        {
            path.Add(PathNode.FromOsmNode(currentNode)!);
            currentNode = currentNode.previousPathNode;
        }
        path.Reverse();

        return path;
    }
}